!                             DRIVER 1
!     --------------------------------------------------------------
!                SIMPLE DRIVER FOR L-BFGS-B (version 2.3)
!     --------------------------------------------------------------

!        L-BFGS-B is a code for solving large nonlinear optimization
!        problems with simple bounds on the variables.

!        The code can also be used for unconstrained problems and is
!        as efficient for these problems as the earlier limited memory
!        code L-BFGS.

!        This is the simplest driver in the package. It uses all the
!        default settings of the code.

!     References:

!        [1] R. H. Byrd, P. Lu, J. Nocedal and C. Zhu, ``A limited
!        memory algorithm for bound constrained optimization'',
!        SIAM J. Scientific Computing 16 (1995), no. 5, pp. 1190--1208.

!        [2] C. Zhu, R.H. Byrd, P. Lu, J. Nocedal, ``L-BFGS-B: FORTRAN
!        Subroutines for Large Scale Bound Constrained Optimization''
!        Tech. Report, NAM-11, EECS Department, Northwestern University, 1994.

!        (Postscript files of these papers are available via anonymous
!        ftp to ece.nwu.edu in the directory pub/lbfgs/lbfgs_bcm.)

!                              *  *  *

!        NEOS, November 1994. (Latest revision April 1997.)
!        Optimization Technology Center.
!        Argonne National Laboratory and Northwestern University.
!        Written by
!                           Ciyou Zhu
!        in collaboration with R.H. Byrd, P. Lu-Chen and J. Nocedal.

!     NOTE: The user should adapt the subroutine 'timer' if 'etime' is
!           not available on the system.  An example for system
!           AIX Version 3.2 is available at the end of this driver.

!     **************

PROGRAM driver

!     This simple driver demonstrates how to call the L-BFGS-B code to solve
!     a sample problem (the extended Rosenbrock function subject to bounds on
!     the variables).  The dimension n of this problem is variable.

USE L_BFGS_B
IMPLICIT NONE

INTEGER, PARAMETER :: nmax  = 1024

!     nmax  is the dimension of the largest problem to be solved.

!     Declare the variables needed by the code.
!     A description of all these variables is given at the end of the driver.

CHARACTER (LEN=60) :: task
INTEGER            :: n, m, iprint, nbd(nmax)
REAL (dp)          :: f, factr, pgtol, x(nmax), l(nmax), u(nmax), g(nmax)

!     Declare a few additional variables for this sample problem.

REAL (dp) :: t1, t2
INTEGER   :: i

!     We wish to have output at every iteration.

iprint = 1

!     We specify the tolerances in the stopping criteria.

factr  = 1.0D+7
pgtol  = 1.0D-5

!     We specify the dimension n of the sample problem and the number
!     m of limited memory corrections stored.  (n and m should not
!     exceed the limits nmax and mmax respectively.)

n      = 25
m      =  5

!     We now provide nbd which defines the bounds on the variables:
!     l   specifies the lower bounds,
!     u   specifies the upper bounds.

!     First set bounds on the odd-numbered variables.

DO i = 1, n, 2
  nbd(i) = 2
  l(i)   = 1.0_dp
  u(i)   = 100._dp
END DO

!     Next set bounds on the even-numbered variables.

DO i = 2, n, 2
  nbd(i) = 2
  l(i)   =-100._dp
  u(i)   = 100._dp
END DO

!     We now define the starting point.

x(1:n) = 3.0_dp

!     We now write the heading of the output.

WRITE (6,16)
16 FORMAT(/ '     Solving sample problem.'  &
          / '      (f = 0.0 at the optimal solution.)' /)

!     We start the iteration by initializing task.

task = 'START'

!     ------- The beginning of the loop ----------

!     This is the call to the L-BFGS-B code.

111 CALL mainlb(n, m, x, l, u, nbd, f, g, factr, pgtol, task, iprint)

IF (task(1:2) == 'FG') THEN
  
!        The minimization routine has returned to request the
!        function f and gradient g values at the current x.
  
!        Compute function value f for the sample problem.
  
  f = .25_dp*(x(1) - 1._dp)**2
  DO i = 2, n
    f = f + (x(i) - x(i-1)**2)**2
  END DO
  f = 4._dp*f
  
!        Compute gradient g for the sample problem.
  
  t1   = x(2) - x(1)**2
  g(1) = 2._dp*(x(1) - 1._dp) - 16._dp*x(1)*t1
  DO i = 2, n-1
    t2   = t1
    t1   = x(i+1) - x(i)**2
    g(i) = 8._dp*t2 - 16._dp*x(i)*t1
  END DO
  g(n) = 8._dp*t1
  
!        Go back to the minimization routine.
  GO TO 111
  
ELSE IF (task(1:5) == 'NEW_X') THEN
  
!        The minimization routine has returned with a new iterate,
!        and we have opted to continue the iteration.
  
  GO TO 111
  
ELSE
  
!        We terminate execution when task is neither FG nor NEW_X.
!        We print the information contained in the string task
!        if the default output is not used and the execution is
!        not stopped intentionally by the user.
  
  IF (iprint <= -1 .AND. task(1:4) /= 'STOP') WRITE(6,*) task
  
END IF

!     ---------- The end of the loop -------------

STOP

END PROGRAM driver

!======================= The end of driver1 ============================
