!                             DRIVER 2
!     --------------------------------------------------------------
!              CUSTOMIZED DRIVER FOR L-BFGS-B (version 2.3)
!     --------------------------------------------------------------

!        L-BFGS-B is a code for solving large nonlinear optimization
!        problems with simple bounds on the variables.

!        The code can also be used for unconstrained problems and is
!        as efficient for these problems as the earlier limited memory
!        code L-BFGS.

!        This driver illustrates how to control the termination of the
!        run and how to design customized output.

!     References:

!        [1] R. H. Byrd, P. Lu, J. Nocedal and C. Zhu, ``A limited
!        memory algorithm for bound constrained optimization'',
!        SIAM J. Scientific Computing 16 (1995), no. 5, pp. 1190--1208.

!        [2] C. Zhu, R.H. Byrd, P. Lu, J. Nocedal, ``L-BFGS-B: FORTRAN
!        Subroutines for Large Scale Bound Constrained Optimization''
!        Tech. Report, NAM-11, EECS Department, Northwestern University,
!        1994.

!        (Postscript files of these papers are available via anonymous
!        ftp to ece.nwu.edu in the directory pub/lbfgs/lbfgs_bcm.)

!                              *  *  *

!        NEOS, November 1994. (Latest revision April 1997.)
!        Optimization Technology Center.
!        Argonne National Laboratory and Northwestern University.
!        Written by
!                           Ciyou Zhu
!        in collaboration with R.H. Byrd, P. Lu-Chen and J. Nocedal.


!     **************

PROGRAM driver

!     This driver shows how to replace the default stopping test
!     by other termination criteria. It also illustrates how to
!     print the values of several parameters during the course of
!     the iteration. The sample problem used here is the same as in
!     DRIVER1 (the extended Rosenbrock function with bounds on the
!     variables).

USE L_BFGS_B
IMPLICIT NONE

INTEGER, PARAMETER :: nmax  = 1024

!     nmax  is the dimension of the largest problem to be solved.

!     Declare the variables needed by the code.
!     A description of all these variables is given at the end of driver1.

CHARACTER (LEN=60) :: task
INTEGER            :: n, m, iprint, nbd(nmax)
REAL (dp)          :: f, factr, pgtol, x(nmax), l(nmax), u(nmax), g(nmax)

!     Declare a few additional variables for the sample problem.

REAL (dp) :: t1, t2
INTEGER   :: i

!     We suppress the default output.

iprint = -1

!     We suppress both code-supplied stopping tests because the
!        user is providing his own stopping criteria.

factr  = 0.0_dp
pgtol  = 0.0_dp

!     We specify the dimension n of the sample problem and the number
!     m of limited memory corrections stored.  (n and m should not
!     exceed the limits nmax and mmax respectively.)

n      = 25
m      =  5

!     We now specify nbd which defines the bounds on the variables:
!     l   specifies the lower bounds,
!     u   specifies the upper bounds.

!     First set bounds on the odd numbered variables.

DO i = 1, n, 2
  nbd(i) = 2
  l(i)   = 1.0_dp
  u(i)   = 100.0_dp
END DO

!     Next set bounds on the even numbered variables.

DO i = 2, n, 2
  nbd(i) = 2
  l(i)   =-100.0_dp
  u(i)   = 100.0_dp
END DO

!     We now define the starting point.

x(1:n) = 3.0_dp

!     We now write the heading of the output.

WRITE (6,16)
16 FORMAT(/ '     Solving sample problem.'  &
          / '      (f = 0.0 at the optimal solution.)' /)

!     We start the iteration by initializing task.

task = 'START'

!     ------- The beginning of the loop ----------

!     This is the call to the L-BFGS-B code.

111 CALL mainlb(n, m, x, l, u, nbd, f, g, factr, pgtol, task, iprint)

IF (task(1:2) == 'FG') THEN
  
!        The minimization routine has returned to request the
!        function f and gradient g values at the current x.
  
!        Compute function value f for the sample problem.
  
  f = .25_dp*(x(1) - 1._dp)**2
  DO i = 2, n
    f = f + (x(i) - x(i-1)**2)**2
  END DO
  f = 4._dp*f
  
!        Compute gradient g for the sample problem.
  
  t1   = x(2) - x(1)**2
  g(1) = 2._dp*(x(1) - 1._dp) - 16._dp*x(1)*t1
  DO i = 2, n-1
    t2   = t1
    t1   = x(i+1) - x(i)**2
    g(i) = 8._dp*t2 - 16._dp*x(i)*t1
  END DO
  g(n) = 8._dp*t1
  
!        Go back to the minimization routine.
  GO TO 111
  
ELSE IF (task(1:5) == 'NEW_X') THEN
  
!        The minimization routine has returned with a new iterate.
!        At this point have the opportunity of stopping the iteration
!        or observing the values of certain parameters
  
!        First are two examples of stopping tests.
  
!        Note: task(1:4) must be assigned the value 'STOP' to terminate
!          the iteration and ensure that the final results are
!          printed in the default format. The rest of the character
!          string task may be used to store other information.
  
!        1) Terminate if the total number of f and g evaluations exceeds 99.
  
  IF (isave(13) >= 99)  &
      task = 'STOP: TOTAL NO. of f AND g EVALUATIONS EXCEEDS LIMIT'
  
!        2) Terminate if |proj g|/(1 + |f|) < 1.0d-10, where
!           "proj g" denoted the projected gradient
  
  IF (dsave(13) <= 1.d-10*(1.0_dp + ABS(f)))  &
      task='STOP: THE PROJECTED GRADIENT IS SUFFICIENTLY SMALL'
  
!        We now wish to print the following information at each iteration:
  
!          1) the current iteration number, isave(9),
!          2) the total number of f and g evaluations, isave(13),
!          3) the value of the objective function f,
!          4) the norm of the projected gradient,  dsave(13)
  
!        See the comments at the end of driver1 for a description
!        of the variables isave and dsave.
  
  WRITE (6, '(2(a, i5, "    "), a, d12.5, "    ", a, d12.5)') ' Iterate', &
         isave(9), 'nfg =', isave(13), 'f = ', f, '|proj g| =', dsave(13)
  
!        If the run is to be terminated, we print also the information
!        contained in task as well as the final value of x.
  
  IF (task(1:4) == 'STOP') THEN
    WRITE (6,*) task
    WRITE (6,*) 'Final X = '
    WRITE (6, '((" ", 6(" ", g12.5)))') x(1:n)
  END IF
  
!        Go back to the minimization routine.
  GO TO 111
  
ELSE
  
!        We terminate execution when task is neither FG nor NEW_X.
!        We print the information contained in the string task
!        if the default output is not used and the execution is
!        not stopped intentionally by the user.
  
  IF (iprint <= -1 .AND. task(1:4) /= 'STOP') WRITE(6,*) task
  
END IF

!           ---------- the end of the loop -------------

STOP

END PROGRAM driver

!======================= The end of driver2 ============================
