!                             DRIVER 3
!     --------------------------------------------------------------
!            TIME-CONTROLLED DRIVER FOR L-BFGS-B (version 2.3)
!     --------------------------------------------------------------

!        L-BFGS-B is a code for solving large nonlinear optimization
!        problems with simple bounds on the variables.

!        The code can also be used for unconstrained problems and is
!        as efficient for these problems as the earlier limited memory
!        code L-BFGS.

!        This driver shows how to terminate a run after some prescribed
!        CPU time has elapsed, and how to print the desired information
!        before exiting.

!     References:

!        [1] R. H. Byrd, P. Lu, J. Nocedal and C. Zhu, ``A limited
!        memory algorithm for bound constrained optimization'',
!        SIAM J. Scientific Computing 16 (1995), no. 5, pp. 1190--1208.

!        [2] C. Zhu, R.H. Byrd, P. Lu, J. Nocedal, ``L-BFGS-B: FORTRAN
!        Subroutines for Large Scale Bound Constrained Optimization''
!        Tech. Report, NAM-11, EECS Department, Northwestern University,
!        1994.

!        (Postscript files of these papers are available via anonymous
!        ftp to ece.nwu.edu in the directory pub/lbfgs/lbfgs_bcm.)

!                              *  *  *

!        NEOS, November 1994. (Latest revision April 1997.)
!        Optimization Technology Center.
!        Argonne National Laboratory and Northwestern University.
!        Written by
!                           Ciyou Zhu
!        in collaboration with R.H. Byrd, P. Lu-Chen and J. Nocedal.


!     **************

PROGRAM driver

!     This time-controlled driver shows that it is possible to terminate
!     a run by elapsed CPU time, and yet be able to print all desired
!     information. This driver also illustrates the use of two
!     stopping criteria that may be used in conjunction with a limit
!     on execution time. The sample problem used here is the same as in
!     driver1 and driver2 (the extended Rosenbrock function with bounds
!     on the variables).

USE L_BFGS_B
IMPLICIT NONE

INTEGER, PARAMETER :: nmax  = 1024

!     nmax  is the dimension of the largest problem to be solved.

!     Declare the variables needed by the code.
!     A description of all these variables is given at the end of driver1.

CHARACTER (LEN=60) :: task
INTEGER            :: n, m, iprint, nbd(nmax)
REAL (dp)          :: f, factr, pgtol, x(nmax), l(nmax), u(nmax), g(nmax)

!     Declare a few additional variables for the sample problem
!     and for keeping track of time.

REAL (dp) :: t1, t2, time1, time2, tlimit
INTEGER   :: i

!     We specify a limite on the CPU time (in seconds).

tlimit = 0.2

!     We suppress the default output.  (The user could also elect to
!     use the default output by choosing iprint >= 0.)

iprint = -1

!     We suppress the code-supplied stopping tests because we will
!     provide our own termination conditions

factr  = 0.0_dp
pgtol  = 0.0_dp

!     We specify the dimension n of the sample problem and the number
!     m of limited memory corrections stored.  (n and m should not
!     exceed the limits nmax and mmax respectively.)

n      = 1000
m      = 10

!     We now specify nbd which defines the bounds on the variables:
!     l   specifies the lower bounds,
!     u   specifies the upper bounds.

!     First set bounds on the odd-numbered variables.

DO i = 1, n, 2
  nbd(i) = 2
  l(i)   = 1.0_dp
  u(i)   = 100.0_dp
END DO

!     Next set bounds on the even-numbered variables.

DO i = 2, n, 2
  nbd(i) = 2
  l(i)   =-100.0_dp
  u(i)   = 100.0_dp
END DO

!     We now define the starting point.

x(1:n) = 3.0_dp

!     We now write the heading of the output.

WRITE (6,16)
16 FORMAT(/ '     Solving sample problem.'  &
          / '      (f = 0.0 at the optimal solution.)' /)

!     We start the iteration by initializing task.

task = 'START'

!     ------- The beginning of the loop ----------

!     We begin counting the CPU time.

CALL timer(time1)

!     This is the call to the L-BFGS-B code.

111 CALL mainlb(n, m, x, l, u, nbd, f, g, factr, pgtol, task, iprint)

IF (task(1:2) == 'FG') THEN
  
!        The minimization routine has returned to request the
!        function f and gradient g values at the current x.
!        Before evaluating f and g we check the CPU time spent.
  
  CALL timer(time2)
  IF (time2 - time1 > tlimit) THEN
    task = 'STOP: CPU EXCEEDING THE TIME LIMIT.'
    
!           Note: Assigning task(1:4)='STOP' will terminate the run;
!           setting task(7:9)='CPU' will restore the information at
!           the latest iterate generated by the code so that it can
!           be correctly printed in the default format.
    
!           In this driver we have chosen to disable the
!           printing options of the code (we set iprint=-1);
!           instead we are using customized output: we print the
!           latest value of x, the corresponding function value f and
!           the norm of the projected gradient |proj g|.
    
!           We print out the information contained in task.
    
    WRITE (6,*) task
    
!           We print the latest iterate contained in wa(j+1:j+n), where
! N.B. Array wa is not available in this version.
    
!    j = 3*n + 2*m*n + 11*m**2
    WRITE (6,*) 'Latest iterate X ='
!    WRITE (6,'((" ", 6(" ", g12.4)))') (wa(i), i = j + 1, j + n)
    WRITE (6,'((" ", 6(" ", g12.4)))') x(1:n)
    
!           We print the function value f and the norm of the projected
!           gradient |proj g| at the last iterate; they are stored in
!           dsave(2) and dsave(13) respectively.
    
    WRITE (6,'(a, g12.5, "    ", a, g12.5)')  &
           ' At latest iterate   f =', dsave(2), '|proj g| =', dsave(13)
    
  ELSE
    
!           The time limit has not been reached and we compute
!           the function value f for the sample problem.
    
    f = .25_dp*(x(1) - 1._dp)**2
    DO i = 2, n
      f = f + (x(i) - x(i-1)**2)**2
    END DO
    f = 4._dp*f
    
!           Compute gradient g for the sample problem.
    
    t1   = x(2) - x(1)**2
    g(1) = 2._dp*(x(1) - 1._dp) - 1.6D1*x(1)*t1
    DO i = 2, n-1
      t2   = t1
      t1   = x(i+1) - x(i)**2
      g(i) = 8._dp*t2 - 1.6D1*x(i)*t1
    END DO
    g(n) = 8._dp*t1
    
  END IF
  
!        Go back to the minimization routine.
  GO TO 111
  
ELSE IF (task(1:5) == 'NEW_X') THEN
  
!        The minimization routine has returned with a new iterate.
!        The time limit has not been reached, and we test whether
!        the following two stopping tests are satisfied:
  
!        1) Terminate if the total number of f and g evaluations exceeds 900.
  
  IF (isave(13) >= 900)  &
      task='STOP: TOTAL NO. of f AND g EVALUATIONS EXCEEDS LIMIT'
  
!        2) Terminate if  |proj g|/(1 + |f|) < 1.0d-10.
  
  IF (dsave(13) <= 1.d-10*(1.0_dp + ABS(f)))  &
      task='STOP: THE PROJECTED GRADIENT IS SUFFICIENTLY SMALL'
  
!        We wish to print the following information at each iteration:
!          1) the current iteration number, isave(9),
!          2) the total number of f and g evaluations, isave(13),
!          3) the value of the objective function f,
!          4) the norm of the projected gradient,  dsve(13)
  
!        See the comments at the end of driver1 for a description
!        of the variables isave and dsave.
  
  WRITE (6,'(2(a, i5, "    "), a, g12.5, "    ", a, g12.5)') ' Iterate',  &
        isave(9), 'nfg =', isave(13), 'f =', f, '|proj g| =', dsave(13)
  
!        If the run is to be terminated, we print also the information
!        contained in task as well as the final value of x.
  
  IF (task(1:4) == 'STOP') THEN
    WRITE (6,*) task
    WRITE (6,*) 'Final X = '
    WRITE (6,'((" ", 6(" ", g11.4)))') x(1:n)
  END IF
  
!        Go back to the minimization routine.
  GO TO 111
  
ELSE
  
!        We terminate execution when task is neither FG nor NEW_X.
!        We print the information contained in the string task
!        if the default output is not used and the execution is
!        not stopped intentionally by the user.
  
  IF (iprint <= -1 .AND. task(1:4) /= 'STOP') WRITE(6,*) task
  
END IF

!     ---------- the end of the loop -------------

STOP

END PROGRAM driver

!======================= The end of driver3 ============================
